# SYNOPSIS
#
#   BASSO_CHECK_CUDA
#
# DESCRIPTION
#
#   Check if user wants to use CUDA and search for the compiler.
#
#   Adds --with-cuda option to configure together with environment variables
#   NVCC and NVCCFLAGS. If NVCC is not given, searches for it. If functional
#   NVCC is present, defines HAVE_CUDA to 1 and sets Automake conditional
#   BASSO_WITH_CUDA to true. 
#

#serial 1

AC_DEFUN([BASSO_CHECK_CUDA],
[AC_ARG_WITH([cuda],
  [AS_HELP_STRING([--with-cuda@<:@=path@:>@],
    [add support for GPU computing with CUDA using NVCC in path])],
  [AS_CASE([$withval], [yes], [basso_cuda_path=no; basso_with_cuda_given=yes],
                       [no],  [],
		              [with_cuda=yes; basso_cuda_path=$withval; basso_with_cuda_given=yes])],
  [with_cuda=yes; basso_cuda_path=no])
  dnl end of AC_ARG_WITH
AC_ARG_VAR([NVCC], [the nvcc compiler])
AC_ARG_VAR([NVCCFLAGS], [the flags for NVCC])

if test "x$with_cuda" != xno; then
dnl Default behaviour is that we try to find NVCC
  if test "x$basso_cuda_path" = xno; then
    basso_cuda_path=$PATH
  fi
  AC_PATH_PROGS([NVCC], [nvcc], [no], [$basso_cuda_path])
  if test "x$NVCC" != xno; then
dnl We have found an NVCC compiler (or it was given as an environmental variable)
dnl Test if it works
    AC_CACHE_CHECK([whether NVCC works], [basso_cv_working_nvcc],
    [basso_cv_working_nvcc=no
      cat > conftest.cu <<EOF
__global__ void foo()
{
  int x = threadIdx.x;
}
EOF
      if $NVCC $NVCCFLAGS -c conftest.cu 2>/dev/null 1>&2
      then
        basso_cv_working_nvcc=yes 
      fi
      rm -f conftest.cu conftest.o
    ])
    dnl END AC_CACHE_CHECK([whether NVCC works],...)
    if test "x$basso_cv_working_nvcc" = xno; then
      with_cuda=no
    fi
  else dnl if test "x$NVCC" != xno
    with_cuda=no
  fi
  if test "x$with_cuda" = xno; then dnl we have set with_cuda to no
    if test "x$basso_with_cuda_given" = xyes; then
      AC_MSG_ERROR([no working nvcc compiler found])
    fi
  fi
fi dnl if test "x$with_cuda" != xno

if test "x$with_cuda" = xyes; then
  AC_DEFINE([HAVE_CUDA], [1], [Define if we have the nvcc compiler])
fi
AM_CONDITIONAL([BASSO_WITH_CUDA], [test "x$with_cuda" = xyes])
])
